/***************************************************************************
 *   Copyright (C) 2008  Antony Martin <martin(dot)antony(at)yahoo(dot)fr> *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 12 octobre 2006
   derniere modification le 16/11/2008 */

#include <SCE/SCEngine.h>

/**
 * \defgroup SCEngine
 */

/** @{ */

/**
 * \defgroup interface Main interface for using the SCEngine
 * \ingroup SCEngine
 * \internal
 * \brief
 */


/**
 * \brief Initializes the SCEngine
 * This function initializes all sub modules of SCEngine.
 * \param outlog stream used to log informations and errors
 * \returns SCE_ERROR if any error has occured, SCE_OK otherwise
 */
int SCE_Init (FILE *outlog, SCEflags flags)
{
    SCE_btstart ();
    /* initialisation des outils */
    if (SCE_Init_Mem () < 0)
    {
        fputs ("error: can't initialize memory manager", stderr);
        SCE_btend ();
        return SCE_ERROR;
    }
    if (outlog)
        Logger_Init (outlog);
    else
    {
        fputs ("error: invalid output stream of errors", stderr);
        SCE_btend ();
        return SCE_ERROR;
    }
    if (SCE_Init_Matrix () < 0)
    {
        Logger_LogFinish ("can't initialize matrices manager");
        SCE_btend ();
        return SCE_ERROR;
    }
    if (SCE_Init_Media () < 0)
    {
        Logger_LogFinish ("can't initialize medias manager");
        SCE_btend ();
        return SCE_ERROR;
    }
    if (SCE_Init_Resource () < 0)
    {
        Logger_LogFinish ("can't initialize resources manager");
        SCE_btend ();
        return SCE_ERROR;
    }
    /***/

    /* initialisation du coeur */
    if (SCE_CInit (flags) < 0)
    {
        Logger_LogFinish ("can't initialize core!");
        SCE_btend ();
        return SCE_ERROR;
    }
    /***/

    /* initialisation des modules de l'interface */
    /* initialisation du gestionnaire de textures */
    if (SCE_Init_Texture () < 0)
    {
        SCE_Quit ();
        Logger_LogFinish ("can't initialize textures manager");
        SCE_btend ();
        return SCE_ERROR;
    }
    /* initialisation du gestionnaire de shaders */
    if (SCE_Init_Shader () < 0)
    {
        SCE_Quit ();
        Logger_LogFinish ("can't initialize shaders manager");
        SCE_btend ();
        return SCE_ERROR;
    }
    /* initialisation du gestionnaire de maillages */
    if (SCE_Init_Mesh () < 0)
    {
        SCE_Quit ();
        Logger_LogFinish ("can't initialize meshs manager");
        SCE_btend ();
        return SCE_ERROR;
    }
    /* initialisation du gestionnaire d'etats */
    if (SCE_Init_State () < 0)
    {
        SCE_Quit ();
        Logger_LogFinish ("can't initialize states manager");
        SCE_btend ();
        return SCE_ERROR;
    }
    /* initialisation du gestionnaire de quads */
    if (SCE_Init_Quad () < 0)
    {
        SCE_Quit ();
        Logger_LogFinish ("can't initialize quads manager");
        SCE_btend ();
        return SCE_ERROR;
    }
    /* initialisation du gestionnaire de quads */
    if (SCE_Init_Scene () < 0)
    {
        SCE_Quit ();
        Logger_LogFinish ("can't initialize scenes manager");
        SCE_btend ();
        return SCE_ERROR;
    }
    /***/

    SCE_btend ();
    return SCE_OK;
}

/**
 * \brief Quit SCEngine and free memory used
 * This function frees every modules initialised by SCE_Init()
 * \returns SCE_ERROR if any error has occured, SCE_OK otherwise
 */
void SCE_Quit (void)
{
    SCE_btstart ();
    SCE_Quit_Scene ();
    SCE_Quit_Quad ();
    SCE_Quit_State ();
    SCE_Quit_Mesh ();
    SCE_Quit_Shader ();
    SCE_Quit_Texture ();
    SCE_CQuit ();
    SCE_Quit_Resource ();
    SCE_Quit_Media ();
    SCE_btend ();
}


const char* SCE_GetVersionString (void)
{
    return SCE_VERSION_STRING;
}

/** @} */
